@@ -26,7 +26,14 @@ sub start {
 
     my ($host, $port) = split /:/, $server;
 
-    # DEBUG "Using port %s", $port;
+    DEBUG "Checking port %s for existing process", $port;
+    if (my $remote = IO::Socket::INET->new (Proto => 'tcp', PeerAddr => $host, PeerPort => $port)) {
+        close $remote;
+        INFO "Already server on %s:%s", $host, $port;
+        exit;
+    }
+
+    DEBUG "Using port %s", $port;
 
     my $pid = fork;
 
@@ -55,10 +62,10 @@ sub wait {
     my $retry = 100;
 
     while ($retry--) {
-        # DEBUG "Checking %s:%s", $host, $port;
+        DEBUG "Checking %s:%s", $host, $port;
         if (my $remote = IO::Socket::INET->new (Proto => 'tcp', PeerAddr => $host, PeerPort => $port)) {
             close $remote;
-            # DEBUG "Connected to %s:%s", $host, $port;
+            DEBUG "Connected to %s:%s", $host, $port;
             return 1;
         }
         else {
@@ -84,7 +91,9 @@ sub stop {
     for my $sig (qw{TERM HUP QUIT INT KILL}) {
         DEBUG "Trying %s", $sig;
         kill ($sig, $pid);
-        if (waitpid ($pid, 0) == $pid) {
+        my $wp = waitpid ($pid, 0);
+        DEBUG "Waitpid returned %s", $wp;
+        if ($wp == $pid or $wp == -1) {
             $result = 1;
             last;
         }
@@ -108,7 +117,7 @@ sub version {
 sub DESTROY {
     my ($self) = @_;
     for my $server (keys %{$self->{servers}}) {
-        # DEBUG "Stopping $server\n";
+        DEBUG "Stopping $server\n";
         $self->stop ($server);
     };
 }
