@@ -2,7 +2,7 @@ package t::Memcached::Mock;
 
 use strict;
 use warnings;
-use Memcached::Client::Log;
+use Memcached::Client::Log qw{DEBUG};
 use Module::Load;
 
 sub new {
@@ -47,7 +47,8 @@ sub add_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->add (@{$tuple});
+        my $value = $self->add (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -69,7 +70,8 @@ sub append_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->append (@{$tuple});
+        my $value = $self->append (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -104,7 +106,8 @@ sub decr_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->decr (@{$tuple});
+        my $value = $self->decr (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -125,7 +128,8 @@ sub delete_multi {
     my ($self, @keys) = @_;
     my (%rv);
     for my $key (@keys) {
-        $rv{$key} = $self->delete ($key);
+        my $value = $self->delete ($key);
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -146,7 +150,11 @@ sub get {
     my $index = $self->{namespace} . (ref $key ? $key->[1] : $key);
     return unless (defined $self->{servers}->{$server});
     DEBUG "M: get: %s - %s", $server, $index;
-    return $self->{servers}->{$server}->{$index};
+    if (length $index > 250) {
+        return undef;
+    } else {
+        return $self->{servers}->{$server}->{$index};
+    }
 }
 
 sub get_multi {
@@ -160,7 +168,7 @@ sub get_multi {
         next unless (defined $self->{servers}->{$server});
         DEBUG "M: get: %s - %s", $server, $index;
         next unless (defined $self->{servers}->{$server}->{$index});
-        $rv{ref $key ? $key->[1] : $key} = $self->{servers}->{$server}->{$index};
+        $rv{ref $key ? $key->[1] : $key} = length $index > 250 ? undef : $self->{servers}->{$server}->{$index};
     }
     return \%rv;
 }
@@ -190,7 +198,8 @@ sub incr_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->incr (@{$tuple});
+        my $value = $self->incr (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -219,7 +228,8 @@ sub prepend_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->prepend (@{$tuple});
+        my $value = $self->prepend (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -241,7 +251,8 @@ sub replace_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->replace (@{$tuple});
+        my $value = $self->replace (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
@@ -253,8 +264,12 @@ sub set {
     my $index = $self->{namespace} . (ref $key ? $key->[1] : $key);
     return 0 unless (defined $self->{servers}->{$server});
     DEBUG "M: set: %s - %s - %s", $server, $index, $value;
-    $self->{servers}->{$server}->{$index} = $value;
-    return 1;
+    if (length $index > 250) {
+        return 0;
+    } else {
+        $self->{servers}->{$server}->{$index} = $value;
+        return 1;
+    }
 }
 
 sub set_multi {
@@ -262,7 +277,8 @@ sub set_multi {
     my (%rv);
     for my $tuple (@{$tuples}) {
         my ($key) = @{$tuple};
-        $rv{$key} = $self->set (@{$tuple});
+        my $value = $self->set (@{$tuple});
+        $rv{$key} = $value if defined $value;
     }
     return \%rv;
 }
